# Generated by cpp11: do not edit by hand

melt_dataframe <- function(data, id_ind, measure_ind, variable_name, value_name, attrTemplate, factorsAsStrings, valueAsFactor, variableAsFactor) {
  .Call(`_tidyr_melt_dataframe`, data, id_ind, measure_ind, variable_name, value_name, attrTemplate, factorsAsStrings, valueAsFactor, variableAsFactor)
}

simplifyPieces <- function(pieces, p, fillLeft) {
  .Call(`_tidyr_simplifyPieces`, pieces, p, fillLeft)
}
